@@ -5,6 +5,7 @@ options(spinner.color="#715D91")
 ui <- fluidPage(theme = "HaDeX_theme.css",
                 #titlePanel("HaDeX"), #: analysis of data from hydrogen deuterium exchange-mass spectrometry"),
                 title = "HaDeX",
+                useShinyjs(),
                 tags$head(includeScript("ga.js"), 
                           tags$link(rel="stylesheet",
                                     href="mobile_version.css",
@@ -37,6 +38,7 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                        h4("Currently HaDeX is limited to `cluster` files from DynamX 3.0 and 2.0"),
                                        h4("Accepted file extensions: .csv, .xsl, .xslx. "),
                                        h4("Supplied file should containt at least two repetitions of the measurement fot the uncertainty to be calculated."),
+                                       h4("If supplied file contains modificated peptides, maximal exchange control cannot be applied."),
                                        #
                                        fluidRow(
                                          column(3, 
@@ -44,7 +46,7 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                                   inputId = "data_file",
                                                   label = "Choose file:",
                                                   multiple = FALSE,
-                                                  accept = c(".csv", ".xslx", ".xsl"),
+                                                  accept = c(".csv", ".xlsx", ".xls"),
                                                   placeholder = "No file selected")),
                                          column(4, 
                                                 h5("File status:"),
@@ -112,15 +114,19 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                          h4("Comparison plot parameters:"),
                                          h5("Choose time parameters:"),
                                          splitLayout(
-                                           selectInput_h(inputId = "in_time",
-                                                         label = "IN",
-                                                         choices = c("0", "1", "5", "25", "1440")),
+                                           div(id = "in_time_part",
+                                               selectInput_h(inputId = "in_time",
+                                                             label = "IN",
+                                                             choices = c("0", "1", "5", "25", "1440"))
+                                               ),
                                            selectInput_h(inputId = "chosen_time",
                                                          label = "CHOSEN",
                                                          choices = c("0", "1", "5", "25", "1440")),
-                                           selectInput_h(inputId = "out_time",
-                                                         label = "OUT",
-                                                         choices = c("0", "1", "5", "25", "1440"))
+                                           div(id = "out_time_part",
+                                               selectInput_h(inputId = "out_time",
+                                                             label = "OUT",
+                                                             choices = c("0", "1", "5", "25", "1440"))
+                                               )
                                          ),
                                          ##
                                          fluidRow(
@@ -232,13 +238,15 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                          tabsetPanel(
                                            tabPanel("Woods plot",
                                                     br(),
-                                                    plotOutput_h("differentialPlot", hover = hoverOpts("differentialPlot_hover", delay = 10, delayType = "debounce")), 
+                                                    div(style = "position:relative",
+                                                      plotOutput_h("differentialPlot", hover = hoverOpts("differentialPlot_hover", delay = 10, delayType = "debounce")), 
+                                                      uiOutput("differentialPlot_debug")),
                                                     downloadButton("differentialPlot_download_button", 
                                                                    "Save chart (.svg)")), 
                                            tabPanel("Data",
                                                     br(),
-                                                    DT::dataTableOutput("differentialPlot_data"))),
-                                         uiOutput("differentialPlot_debug")
+                                                    DT::dataTableOutput("differentialPlot_data")))
+                                         
                                          
                                        )
                               ),
@@ -266,7 +274,8 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                          tabsetPanel(
                                            tabPanel("Peptide Coverage",
                                                     br(),
-                                                    withSpinner(plotOutput("stateOverlap")),
+                                                    withSpinner(plotOutput("stateOverlap", hover = hoverOpts("stateOverlap_hover", delay = 10, delayType = "debounce"))),
+                                                    uiOutput("stateOverlap_debug"), 
                                                     downloadButton("stateOverlap_download_button",
                                                                    "Save chart (.svg)")),
                                            
@@ -278,7 +287,9 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                          tabsetPanel(
                                            tabPanel("Position Frequency",
                                                     br(),
-                                                    withSpinner(plotOutput("stateOverlapDist")),
+                                                    div(style = "position:relative;",
+                                                        withSpinner(plotOutput("stateOverlapDist", hover = hoverOpts("stateOverlapDist_hover", delay = 10, delayType = "debounce"))),
+                                                        uiOutput("stateOverlapDist_debug")),
                                                     downloadButton("stateOverlapDist_download_button",
                                                                    "Save chart (.svg)")
                                            ),
@@ -315,14 +326,15 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                                )
                                              )
                                            ),
-                                           mainPanel(withSpinner(plotOutput("aminoDist")),
+                                           mainPanel(withSpinner(plotOutput("aminoDist", hover = hoverOpts("aminoDist_hover", delay = 10, delayType = "debounce"))),
+                                                     uiOutput("aminoDist_debug"), 
                                                      downloadButton("aminoDist_download_button",
                                                                     "Save chart (.svg)"),
-                                                     p("Source: Kyte, J., and Doolittle, R.F. (1982). A simple method for displaying the hydropathic character of a protein. J Mol Biol 157, 105–132.")),
+                                                     p("Source: Kyte, J., and Doolittle, R.F. (1982). A simple method for displaying the hydropathic character of a protein. J Mol Biol 157, 105–132."))
                                          )
                                        )
                               ),
-                              tabPanel("Kinetics",
+                              tabPanel("Uptake curves",
                                        br(),
                                        sidebarPanel(
                                          class = "scrollable",
@@ -334,14 +346,19 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                                         label = "Choose values type:",
                                                         choices = c("relative", "absolute"),
                                                         selected = "relative"),
-                                         h5("Choose time parameters:"),
-                                         splitLayout(
-                                           selectInput_h(inputId = "kin_in_time",
-                                                         label = "IN",
-                                                         choices = c("0", "1", "5", "25", "1440")),
-                                           selectInput_h(inputId = "kin_out_time",
-                                                         label = "OUT",
-                                                         choices = c("0", "1", "5", "25", "1440"))),
+                                         div(id = "kin_time_part", 
+                                           h5("Choose time parameters:"),
+                                           splitLayout(
+                                             div(id = "kin_in_time_part",
+                                                 selectInput_h(inputId = "kin_in_time",
+                                                               label = "IN",
+                                                               choices = c("0", "1", "5", "25", "1440"))
+                                                 ),
+                                             div(id = "kin_out_time_part",
+                                                 selectInput_h(inputId = "kin_out_time",
+                                                               label = "OUT",
+                                                               choices = c("0", "1", "5", "25", "1440")))
+                                                 )),
                                          h5("Choose peptide:"),
                                          dataTableOutput_h("peptide_list_data"), ## !! 
                                          actionButton(inputId = "reset_peptide_list", 
@@ -350,7 +367,7 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                          br(),
                                          br(),
                                          sliderInput(inputId = 'kin_plot_y_range',
-                                                     label = 'Choose y range for kinetic plot:',
+                                                     label = 'Choose y range for the plot:',
                                                      min = -50,
                                                      max = 200,
                                                      value = c(-10, 100),
@@ -363,22 +380,23 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                            class = "hideable",
                                            id = "kin-labs",
                                            textInput(inputId = "kin_plot_title",
-                                                     label = "Kinetic plot title:",
-                                                     value = "Kinetic plot for chosen peptides"),
+                                                     label = "Uptake curve title:",
+                                                     value = "Uptake curve for chosen peptides"),
                                            textInput(inputId = "kin_plot_x_label",
-                                                     label = "Kinetic plot axis x label:",
+                                                     label = "Uptake curve axis x label:",
                                                      value = "Time point [min]"),
                                            textInput(inputId = "kin_plot_y_label",
-                                                     label = "Kinetic plot axis y label:",
+                                                     label = "Uptake curve axis y label:",
                                                      value = "Deuteration")
                                          )
                                        ),
                                        mainPanel(
                                          tabsetPanel(
                                            tabPanel(
-                                             "Kinetic plot",
+                                             "Uptake curve",
                                              br(),
-                                             plotOutput_h("kinetic_plot_chosen_peptides", height = 600),
+                                             plotOutput_h("kinetic_plot_chosen_peptides", hover = hoverOpts("kinetic_plot_chosen_peptides_hover", delay = 10, delayType = "debounce"), height = 600),
+                                             uiOutput("kinetic_plot_chosen_peptides_debug"), 
                                              downloadButton("kineticPlot_download_button", 
                                                             "Save chart (.svg)")),
                                            tabPanel("Data", 
@@ -410,7 +428,8 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                          tabsetPanel(
                                            tabPanel("Quality control plot", 
                                                     br(),
-                                                    plotOutput_h("quality_control_plot", height = 600),
+                                                    plotOutput_h("quality_control_plot", hover = hoverOpts("quality_control_plot_hover", delay = 10, delayType = "debounce"), height = 600),
+                                                    uiOutput("quality_control_plot_debug"), 
                                                     downloadButton("quality_control_plot_download_button", 
                                                                    "Save chart (.svg)"),
                                                     br(),
@@ -454,10 +473,10 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                                                            label = "Theoretical Woods Plot",
                                                                            value = TRUE),
                                                              checkboxInput(inputId = "export_kin_plot",
-                                                                           label = "Kinetic Plot",
+                                                                           label = "Uptake Curve",
                                                                            value = FALSE),
                                                              checkboxInput(inputId = "export_theo_kin_plot",
-                                                                           label = "Theoretical Kinetic Plot", 
+                                                                           label = "Theoretical Uptake Curve", 
                                                                            value = FALSE),
                                                              checkboxInput(inputId = "export_quality_control_plot",
                                                                            label = "Quality Control Plot",
@@ -476,9 +495,9 @@ ui <- fluidPage(theme = "HaDeX_theme.css",
                                                              checkboxInput(inputId = "export_theo_woods_plot_data",
                                                                            label = "Theoretical Woods Plot Data"),
                                                              checkboxInput(input = "export_kin_plot_data", 
-                                                                           label = "Kinetic Plot Data"),
+                                                                           label = "Uptake Curve Data"),
                                                              checkboxInput(inputId = "export_theo_kin_plot_data", 
-                                                                           label = "Theoretical Kinetic Plot Data"),
+                                                                           label = "Theoretical Uptake Curve Data"),
                                                              checkboxInput(inputId = "export_quality_control_plot_data",
                                                                            label = "Quality Control Plot Data"))),
                                                     br(),
